use axum::{
    extract::{Json, Request},
    Router,
};
use clap::Parser;
use serde::Serialize;
use std::collections::BTreeMap;

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
struct CliArgs {
    /// Port to run on.
    #[arg(short, long, default_value_t = 8080)]
    port: u16,

    /// Listen to IP mask
    #[arg(short, long, default_value = "0.0.0.0")]
    ips: String,
}

#[derive(Serialize)]
struct EchoHeader {
    name: String,
    value: String,
}

#[derive(Serialize)]
struct EchoResponse {
    method: String,
    path: String,
    host: String,
    headers: BTreeMap<String, String>,
}

impl EchoResponse {
    fn new(req: &Request) -> Self {
        let req_uri = req.uri();
        let mut headers = BTreeMap::new();
        for (header_name, header_value) in req.headers().iter() {
            headers.insert(
                header_name.to_string(),
                header_value.to_str().unwrap_or("ERROR").to_string(),
            );
        }
        EchoResponse {
            method: req.method().to_string(),
            path: req_uri.path().to_string(),
            host: req_uri.host().unwrap_or("").to_string(),
            headers,
        }
    }
}

async fn echo_request(request: Request) -> Json<EchoResponse> {
    Json(EchoResponse::new(&request))
}

#[tokio::main]
async fn main() {
    let cli_args = CliArgs::parse();
    let listen_on = (cli_args.ips, cli_args.port);
    let listen_on = format!("{ip}:{port}", ip = listen_on.0, port = listen_on.1);

    let app = Router::new().fallback(echo_request);
    let listener = tokio::net::TcpListener::bind(&listen_on)
        .await
        .expect(&format!("Attempted binding to {}", listen_on));
    axum::serve(listener, app)
        .await
        .expect("Server should start");
}
